/*
 * Copyright (c) 2001 - 2003 Bernd Walter Computer Technology
 * http://www.bwct.de
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $URL: https://devel.bwct.de/svn/projects/modbus/mb_ibuttond/tags/0.2/mb_ibuttond.cc $
 * $Date: 2004-11-09 19:39:26 +0100 (Tue, 09 Nov 2004) $
 * $Author: ticso $
 * $Rev: 1793 $
 */

#include <bwctmb/bwctmb.h>

int main(int argc, char *argv[]);
void usage(void);

static uint8_t
calc_crc(uint8_t *buff, uint8_t num_vals)
{
	uint8_t shift_reg, data_bit, sr_lsb, fb_bit, i, j;

	shift_reg = 0;
	for (i = 0; i < num_vals; i++) { // for each byte
		for (j = 0; j < 8; j++) { // for each bit
			data_bit = (buff[i] >> j) &0x01;
			sr_lsb = shift_reg & 0x01;
			fb_bit = (data_bit ^ sr_lsb) & 0x01;
			shift_reg = shift_reg >> 1;
			if (fb_bit)
				shift_reg = shift_reg ^ 0x8c;
		}
	}
	return shift_reg;
}

int
main(int argc, char *argv[]) {
	uint8_t address;
	int i;
	FILE *execpipe;
	char *readprog;
	uint8_t nibble;
	uint8_t field[8];
	uint8_t crc;
	uint16_t touch_reset = 0;
	uint16_t ser_reg = 0;
	String host;
	String port;

	if (argc < 5)
		usage();
	if (argc > 7)
		usage();
	host = argv[1];
	port = argv[2];
	argc -= 3;
	argv += 3;

	Modbus mb(host, port);

	address = atol(argv[0]);
	readprog = argv[1];
	if (argc > 2)
		touch_reset = atol(argv[2]);
	if (argc > 3)
		ser_reg = atol(argv[3]);

	daemon(0, 0);

	for (;;) {
		try {
begin:
			usleep(500000);
			if (mb.read_discrete_input(address, touch_reset))
				goto begin;

			SArray<uint16_t> data;
			data = mb.read_input_registers(address, ser_reg, 4);
			for (i = 3; i >= 0; i--) {
				field[i * 2] = data[i] & 0xff;
				field[i * 2 + 1] = (data[i] >> 8) & 0xff;
			}

			crc = calc_crc(&field[0], 7);
			if (crc != field[7]) {
				goto begin;
			}

			execpipe = popen(readprog, "w");
			if (execpipe == NULL) {
				goto begin;
			}

			for (i = 7; i >= 0; i--) {
				nibble = (field[i] >> 4) & 0x0f;
				fprintf(execpipe, "%c", (nibble > 9) ? 'a' - 10 + nibble : '0' + nibble);
				nibble = field[i] & 0x0f;
				fprintf(execpipe, "%c", (nibble > 9) ? 'a' - 10 + nibble : '0' + nibble);
			}

			fprintf(execpipe, "\n");
			pclose(execpipe);

		} catch (std::exception& e) {
			// exception thrown is logged anyway
			sleep (3);
		}
	}
	return 0;
}

void
usage(void) {

	printf("usage: mb_ibuttond ip port slaveaddress script [touch_reset] [ser_reg]\n");
	exit(1);
}

